/***** Changes ***********
 * Date					Edit				Bug					Description
 * 02-Jun-17		jmg0594									Added setting to specify Push Notification Server.
 *
 */

package net.omnis.android.notifications

import android.content.*
import android.content.pm.PackageManager
import android.os.AsyncTask
import android.os.Build
import android.util.Log
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.android.volley.AuthFailureError
import com.android.volley.Request
import com.android.volley.Response
import com.android.volley.toolbox.StringRequest
import com.android.volley.toolbox.Volley
import com.google.firebase.FirebaseApp
import com.google.firebase.iid.FirebaseInstanceId
import net.omnis.android.settings.OMSettings
import net.omnis.omnisinterface.OMDialogs
import net.omnis.omnisinterface.OmnisInterface
import org.json.JSONObject
import java.io.IOException

/**
 * Created by jgissing on 14/12/2016.
 */

class PushRegistrationManager(private val mContext: Context) {

	// Our handler for received Intents. This will be called whenever an Intent
	// with an action named "custom-event-name" is broadcasted.
	private val mMessageReceiver = object : BroadcastReceiver() {
		override fun onReceive(context: Context, intent: Intent) {
			if (intent.action == "omnis-firebase-token-renewed" && pushNotificationsEnabled)
				sendToken()
		}
	}

	private var tokenSendPending: Boolean
		get() {
			val prefs = mContext.getSharedPreferences("PUSH_SETTINGS", Context.MODE_PRIVATE)
			return prefs.getBoolean(PREF_TOKEN_PENDING, false)
		}
		set(pending) {
			val prefs = mContext.getSharedPreferences("PUSH_SETTINGS", Context.MODE_PRIVATE)
			val editor = prefs.edit()
			editor.putBoolean(PREF_TOKEN_PENDING, pending)
			editor.apply()
		}

	private var pushNotificationsEnabled: Boolean
		get() {
			val prefs = mContext.getSharedPreferences("PUSH_SETTINGS", Context.MODE_PRIVATE)
			return prefs.getBoolean(PREF_NOTIFICATIONS_ENABLED, false)
		}
		set(enabled) {
			val prefs = mContext.getSharedPreferences("PUSH_SETTINGS", Context.MODE_PRIVATE)
			val editor = prefs.edit()
			editor.putBoolean(PREF_NOTIFICATIONS_ENABLED, enabled)
			editor.apply()
		}

	init {


		/* Register to receive messages when the Firebase Token is renewed:
		Do this regardless of whether push notifications are currently enabled,
		as if they are later enabled, we may not be notified of a new token.
		*/
		LocalBroadcastManager.getInstance(mContext).registerReceiver(mMessageReceiver,
					IntentFilter("omnis-firebase-token-renewed"))

		if (pushNotificationsEnabled) {
			enablePushNotifications(true)

			// If we previously failed to send the firebase token to the server, try again now:
			if (tokenSendPending)
				sendToken()
		}
	}

	fun enablePushNotifications(enable: Boolean) {

		// Enable/disable the Firebase Messaging Service:
		val pm = mContext.packageManager
		val componentName = ComponentName(mContext, OmnisFirebaseMessagingService::class.java)
		pm.setComponentEnabledSetting(componentName,
					if (enable) PackageManager.COMPONENT_ENABLED_STATE_ENABLED else PackageManager.COMPONENT_ENABLED_STATE_DISABLED,
					PackageManager.DONT_KILL_APP)

		if (enable == pushNotificationsEnabled)
			return  // Nothing more to do, as not changing enabled state.

		// If we just enabled notifications, send the token (in case notifications were disabled when we first got the token).
		if (enable) {
			if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O)
				OmnisNotificationChannels.addChannelsFromSettings(mContext)

			sendToken()
		}
		else { // We've disabled notifications - delete the instanceID, so Firebase doesn't try to send this new notifications.
			try {
				FirebaseInstanceId.getInstance().deleteInstanceId()
			}
			catch (e: IOException) {
				OMDialogs.showOKMessage(mContext, e.localizedMessage, "Error")
			}
		}

		pushNotificationsEnabled = enable
	}

	fun deregister() {
		// Unregister since the activity is about to be closed.
		LocalBroadcastManager.getInstance(mContext).unregisterReceiver(mMessageReceiver)
	}

	private fun sendToken() {
		tokenSendPending = true
		// Get updated InstanceID token.
		FirebaseInstanceId.getInstance().instanceId.addOnSuccessListener { instanceIdResult ->
			SendTokenTask().execute(instanceIdResult.token)
		}

	}

	private fun sendRegistrationToServer(token: String) {

		val pushServerUrl = "${OMSettings.getSettingString(OMSettings.IDs.PUSH_NOTIFICATION_SERVER)}/api/pushnotifications/pushnotifications/register" // jmg0594

		val groupsMask = OMSettings.getSettingInt(OMSettings.IDs.PUSH_NOTIFICATION_GROUPMASK)

		// Don't seem to be able to get the Project ID directly. This seems to be the first part of storageBucket though, so get it via that:
		val storageBucket = FirebaseApp.getInstance()!!.options.storageBucket
		val projectName = storageBucket.substring(0, storageBucket.indexOf("."))

		// Instantiate the RequestQueue.
		val queue = Volley.newRequestQueue(mContext)


		// Send a POST to the Omnis server
		val stringRequest = object : StringRequest(Request.Method.POST, pushServerUrl,
					Response.Listener { response ->
						if (response == "OK")
							tokenSendPending = false
					},
					Response.ErrorListener { Log.e("VOLLEY ERROR", "Error") }
		) {

			override fun getBodyContentType(): String {
				return "application/json"
			}

			@Throws(AuthFailureError::class)
			override fun getBody(): ByteArray? {
				val data = JSONObject()
				try {
					data.put("HWID", OmnisInterface.DEVICE_ID)
					data.put("TOKEN", token)
					data.put("OSType", 1) // Type 1 = Android
					data.put("AppGroup", projectName)
					data.put("Groups", groupsMask)

					return data.toString().toByteArray(charset("UTF-8"))
				}
				catch (e: Exception) {
					return null
				}

			}
		}

		queue.add(stringRequest)
	}


	private inner class SendTokenTask : AsyncTask<String, Void, Void>() {

		override fun doInBackground(vararg params: String): Void? {
			sendRegistrationToServer(params[0])
			return null
		}
	}

	companion object {

		private val PREF_NOTIFICATIONS_ENABLED = "NOTIFICATIONS_ENABLED"
		private val PREF_TOKEN_PENDING = "TOKEN_PENDING"
	}

}
