package net.omnis.android.ui.main

import android.content.res.Configuration
import android.os.Build
import android.view.GestureDetector
import android.view.MotionEvent
import android.view.View
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.constraintlayout.widget.ConstraintSet
import androidx.core.view.ViewCompat
import net.omnis.android.R
import net.omnis.android.settings.OMSettings

abstract class OMImmersiveFragment: androidx.fragment.app.Fragment() {

	private var mImmersive = false
	private var mShowingImmersiveOverlay = false
	private var mOverlayID = 0
	private var mInMultiWindowMode = false


	fun checkImmersiveMode() {
		mImmersive = OMSettings.getSettingBool(OMSettings.IDs.IMMERSIVE_MODE) ?: false

		mInMultiWindowMode = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N)
			activity?.isInMultiWindowMode ?: false
		else
			false

		if (isImmersive())
			enableImmersiveMode()
		else
			disableImmersiveMode()
	}

	private fun enableImmersiveMode() {
		mImmersive = true

		if (isImmersive()) {
			hideSystemUI()

			view?.setOnSystemUiVisibilityChangeListener { visibility ->
				if (visibility == 0 && !mShowingImmersiveOverlay)
					addImmersiveOverlay()
				else if (visibility != 0 && mShowingImmersiveOverlay)
					removeImmersiveOverlay()
			}
		}
	}

	private fun disableImmersiveMode() {
		mImmersive = false
		removeImmersiveOverlay()
		activity?.window?.decorView?.systemUiVisibility = View.SYSTEM_UI_FLAG_VISIBLE
	}

	private fun addImmersiveOverlay() {
		val layout = view?.findViewById<ConstraintLayout>(R.id.main)
		if (layout != null) {
			// Create an overlay view:
			val overlay = View(context)
			mOverlayID = ViewCompat.generateViewId()
			overlay.id = mOverlayID
			overlay.isClickable = true
			layout.addView(overlay)

			// Set its constraints:
			val constraints = ConstraintSet()
			constraints.connect(mOverlayID, ConstraintSet.TOP, ConstraintSet.PARENT_ID, ConstraintSet.TOP)
			constraints.connect(mOverlayID, ConstraintSet.LEFT, ConstraintSet.PARENT_ID, ConstraintSet.LEFT)
			constraints.connect(mOverlayID, ConstraintSet.RIGHT, ConstraintSet.PARENT_ID, ConstraintSet.RIGHT)
			constraints.connect(mOverlayID, ConstraintSet.BOTTOM, ConstraintSet.PARENT_ID, ConstraintSet.BOTTOM)
			constraints.applyTo(layout)

			mShowingImmersiveOverlay = true

			// Add a touchdown listener to hide the UI again:
			val gestureDetector = GestureDetector(activity, object : GestureDetector.SimpleOnGestureListener() {
				override fun onDown(e: MotionEvent?): Boolean {
					hideSystemUI()
					return super.onSingleTapUp(e)
				}
			})
			overlay.setOnTouchListener { _, event -> gestureDetector.onTouchEvent(event) }
		}
	}

	private fun removeImmersiveOverlay() {
		val layout = view?.findViewById<ConstraintLayout>(R.id.main)
		if (layout != null) {
			val overlay = layout.findViewById<View>(mOverlayID)
			if (overlay != null)
				layout.removeView(overlay)

			mShowingImmersiveOverlay = false
		}
	}



	private fun hideSystemUI() {
		// Enables regular immersive mode.
		// For "lean back" mode, remove SYSTEM_UI_FLAG_IMMERSIVE.
		// Or for "sticky immersive," replace it with SYSTEM_UI_FLAG_IMMERSIVE_STICKY
		activity?.window?.decorView?.systemUiVisibility = (
					View.SYSTEM_UI_FLAG_IMMERSIVE
					// Set the content to appear under the system bars so that the
					// content doesn't resize when the system bars hide and show.
					or View.SYSTEM_UI_FLAG_LAYOUT_STABLE
					or View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
					or  View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
					// Hide the nav bar and status bar
					or View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
					or View.SYSTEM_UI_FLAG_FULLSCREEN)

		removeImmersiveOverlay()
	}

	// Shows the system bars by removing all the flags
	// except for the ones that make the content appear under the system bars.
	private fun showSystemUI() {
		activity?.window?.decorView?.systemUiVisibility = (
					View.SYSTEM_UI_FLAG_LAYOUT_STABLE
					or View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
					or View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN)
	}


	fun onWindowFocusChanged(hasFocus: Boolean) {
		if (hasFocus && isImmersive())
			hideSystemUI()
	}

	private fun isImmersive(): Boolean {
		return mImmersive && !mInMultiWindowMode
	}

	override fun onConfigurationChanged(newConfig: Configuration?) {
		super.onConfigurationChanged(newConfig)

		mInMultiWindowMode = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N)
			activity?.isInMultiWindowMode ?: false
		else
			false

		checkImmersiveMode()
	}

}